import TextDatasource from "./TextDatasource";
import Credentials from "../credentials/Credentials";
import MyButtercupClient from "../myButtercup/MyButtercupClient";
import { BufferLike, DatasourceLoadedData, EncryptedContent, History, VaultID, VaultInsights } from "../types";
interface OnTokensUpdatedCallback {
    (): void;
}
/**
 * My Buttercup datasource
 * @augments TextDatasource
 * @memberof module:Buttercup
 */
export default class MyButtercupDatasource extends TextDatasource {
    _client: MyButtercupClient;
    _clientID: string;
    _clientSecret: string;
    _onTokensUpdated: OnTokensUpdatedCallback;
    _updateID: number;
    _vaultID: number;
    accessToken: string;
    refreshToken: string;
    /**
     * Constructor for the datasource
     * @param credentials Credentials for the datasource
     * @memberof MyButtercupDatasource
     */
    constructor(credentials: Credentials);
    /**
     * My Buttercup client
     * @readonly
     * @memberof MyButtercupDatasource
     */
    get client(): MyButtercupClient;
    /**
     * Change the master password for the vault
     * (Called with a preflight check to ensure that the datasource is
     * ready to change - if not ready, the method will return false)
     * @param newCredentials
     * @param preflight
     * @returns A promise that resolves
     *  with a boolean value during preflight, or with simply undefined
     *  if performing the final change action.
     * @example
     *  const creds = Credentials.fromPassword("test");
     *  const isReady = await tds.changePassword(
     *      creds,
     *      true // preflight
     *  );
     *  if (!isReady) {
     *      throw new Error("Datasource unable to change password");
     *  }
     *  await tds.changePassword(creds, false);
     * @memberof MyButtercupDatasource
     */
    changePassword(newCredentials: Credentials, preflight: boolean): Promise<boolean | undefined>;
    /**
     * Get attachment buffer
     * - Loads the attachment contents into a buffer
     * @param vaultID The ID of the vault
     * @param attachmentID The ID of the attachment
     * @returns The attachment data
     * @memberof MyButtercupDatasource
     */
    getAttachment(vaultID: VaultID, attachmentID: string): Promise<BufferLike>;
    /**
     * Get the available storage space, in bytes
     * @returns {Number|null} Bytes of free space, or null if not
     *  available
     * @memberof MyButtercupDatasource
     */
    getAvailableStorage(): Promise<number>;
    /**
     * Get the total storage space, in bytes
     * @returns {Number|null} Bytes of free space, or null if not
     *  available
     * @memberof MyButtercupDatasource
     */
    getTotalStorage(): Promise<number>;
    /**
     * Load vault history from remote
     * @param credentials The archive credentials
     * @returns Promise which resolves with vault data
     * @memberof MyButtercupDatasource
     */
    load(credentials: Credentials): Promise<DatasourceLoadedData>;
    /**
     * Override for history difference checking
     * @see VaultSource#localDiffersFromRemote
     * @param masterCredentials Master service credentials
     * @param archiveHistory Archive history lines
     * @returns True if differing, false otherwise
     * @memberof MyButtercupDatasource
     */
    localDiffersFromRemote(masterCredentials: Credentials, archiveHistory: History): Promise<boolean>;
    /**
     * Put attachment data
     * @param {String} vaultID The ID of the vault
     * @param {String} attachmentID The ID of the attachment
     * @param {Buffer|ArrayBuffer} buffer The attachment data
     * @param {Object} details
     * @returns {Promise}
     * @memberof MyButtercupDatasource
     */
    putAttachment(vaultID: any, attachmentID: any, buffer: any, details: any): Promise<void>;
    /**
     * Remove an attachment
     * @param {String} vaultID The ID of the vault
     * @param {String} attachmentID The ID of the attachment
     * @returns {Promise}
     * @memberof MyButtercupDatasource
     */
    removeAttachment(vaultID: any, attachmentID: any): Promise<void>;
    /**
     * Save vault contents to remote
     * @param history The vault history lines
     * @param credentials Vault credentials
     * @memberof MyButtercupDatasource
     */
    save(history: History, credentials: Credentials): Promise<EncryptedContent>;
    /**
     * Whether or not the datasource supports attachments
     * @returns {Boolean}
     * @memberof MyButtercupDatasource
     */
    supportsAttachments(): boolean;
    /**
     * Whether or not the datasource supports the changing of the master password
     * @returns True if it supports changing the master password
     * @memberof MyButtercupDatasource
     */
    supportsPasswordChange(): boolean;
    /**
     * Update vault/account insights
     * @param insights Insights to update
     * @memberof MyButtercupDatasource
     */
    updateInsights(insights: VaultInsights): Promise<void>;
    /**
     * Update the OAuth2 tokens
     * @param accessToken The access token
     * @param refreshToken The refresh token
     * @memberof MyButtercupDatasource
     */
    updateTokens(accessToken: string, refreshToken: string, updateClientTokens?: boolean): void;
    /**
     * Create a new MyButtercupClient instance and attach
     * event listeners
     * @protected
     * @memberof MyButtercupDatasource
     * @fires MyButtercupDatasource#updatedClient
     */
    _createNewClient(): void;
}
export {};
