/// <reference types="node" />
import EventEmitter from "eventemitter3";
import { CowlError, MyButtercupAttachment, MyButtercupAttachmentDetails, MyButtercupDigest, MyButtercupEncryptedShare, MyButtercupUsersListItem, MyButtercupVaultDetails, VaultInsights } from "../types";
export interface MyButtercupFetchedShares {
    [shareID: string]: MyButtercupEncryptedShare;
}
/**
 * My Buttercup client
 * @augments EventEmitter
 * @memberof module:Buttercup
 */
export default class MyButtercupClient extends EventEmitter {
    /**
     * Exchange an auth code for tokens
     * @param authCode OAuth2 auth code, retrieved from browser-
     *  based OAuth2 flow using a user's username and password
     * @param clientID The OAuth2 client ID
     * @param clientSecret The OAuth2 client secret
     * @param redirectURI The OAuth2 client redirect URI
     * @memberof MyButtercupClient
     * @static
     */
    static exchangeAuthCodeForTokens(authCode: string, clientID: string, clientSecret: string, redirectURI: string): Promise<{
        accessToken: string;
        refreshToken: string;
    }>;
    /**
     * Generate an OAuth2 authorisation URL using the client ID of the current
     * application platform (eg. Buttercup browser extension)
     * @param clientID The OAuth2 client ID registered on
     *  my.buttercup.pw
     * @returns The generated URL
     * @memberof MyButtercupClient
     * @static
     */
    static generateAuthorisationURL(clientID: string): string;
    _accessToken: string;
    _clientID: string;
    _clientSecret: string;
    _lastDigest: MyButtercupDigest;
    _lastDigestTime: number;
    _refreshToken: string;
    request: any;
    /**
     * Create a new client instance
     * @param clientID The client identifier
     * @param clientSecret The client secret
     * @param accessToken Access token
     * @param refreshToken Refresh token
     */
    constructor(clientID: string, clientSecret: string, accessToken: string, refreshToken: string);
    /**
     * The current access token
     * @readonly
     */
    get accessToken(): string;
    /**
     * The last client digest response
     * @readonly
     */
    get digest(): MyButtercupDigest;
    /**
     * The refresh token
     * @readonly
     */
    get refreshToken(): string;
    changePassword(password: string, passwordToken: string): Promise<void>;
    deleteAttachment(attachmentID: string): Promise<void>;
    fetchAttachment(attachmentID: string): Promise<MyButtercupAttachment>;
    fetchAttachmentDetails(attachmentID: string): Promise<MyButtercupAttachmentDetails>;
    /**
     * Fetch user shares
     * @param ids Share IDs
     * @memberof MyButtercupClient
     */
    fetchShares(ids: Array<string>): Promise<MyButtercupFetchedShares>;
    /**
     * Fetch user vault contents
     * @returns The user's vault contents
     * @memberof MyButtercupClient
     */
    fetchUserVault(): Promise<{
        archive: string;
        updateID: number;
    }>;
    /**
     * Fetch the user's vault details
     * @returns The details of the vault
     * @memberof MyButtercupClient
     */
    fetchUserVaultDetails(): Promise<MyButtercupVaultDetails>;
    /**
     * Fetch and set account digest information
     * @returns Digest information
     * @memberof MyButtercupClient
     */
    retrieveDigest(): Promise<MyButtercupDigest>;
    /**
     * Get the list of users available to address for the user
     * @memberof MyButtercupClient
     */
    retrieveUsersList(): Promise<Array<MyButtercupUsersListItem>>;
    /**
     * Get the list of users for an organisation
     * (User must be present in organisation, or this method will fail)
     * @param orgID The ID of the organisation
     * @memberof MyButtercupClient
     */
    retrieveUsersListForOrganisation(orgID: number): Promise<Array<MyButtercupUsersListItem>>;
    /**
     * Test if a password token is valid
     * @param passwordToken The password change token
     * @memberof MyButtercupClient
     */
    testPasswordChange(passwordToken: string): Promise<void>;
    /**
     * Update the digest if required
     * @memberof MyButtercupClient
     */
    updateDigestIfRequired(): Promise<void>;
    /**
     * Upload an attachment
     * @param id The attachment ID
     * @param name The attachment name
     * @param type The attachment MIME type
     * @param data Encrypted attachment data
     */
    uploadAttachment(id: string, name: string, type: string, data: Buffer | ArrayBuffer): Promise<any>;
    /**
     * Write insights to the remote account
     * @param insights The insights data
     * @memberof MyButtercupClient
     */
    writeInsights(insights: VaultInsights): Promise<void>;
    /**
     * Write the user vault contents back to the server
     * @param contents Encrypted vault contents
     * @param previousUpdateID The previous update ID received from the server
     * @param newUpdateID The new update ID to set after a successful write
     * @returns A promise that resolves once the write has been completed
     * @memberof MyButtercupClient
     */
    writeUserArchive(contents: string, previousUpdateID: number, newUpdateID: number): Promise<void>;
    /**
     * Handle a request failure (processes token expiration etc.)
     * @param err The received error from making a request
     * @throws {Error} Throws if the error was not catchable
     * @returns Returns a promise if an action can be taken
     *  to remedy the situation
     * @memberof MyButtercupClient
     * @protected
     */
    _handleRequestFailure(err: CowlError): Promise<void>;
    /**
     * Refresh tokens
     * @memberof MyButtercupClient
     * @protected
     * @fires MyButtercupClient#tokensUpdated
     */
    _performTokenRefresh(): Promise<void>;
}
